require 'mysql'

module Embulk
  module Plugin
    module MySQL

      class NotSupportedSchema < Exception; end

      class OutputMysql < OutputPlugin
        warn '[DEPRECATION] `embulk-plugin-mysql` is deprecated. Please use `embulk-output-mysql` instead.'
        Plugin.register_output('mysql', self)

        # TODO: use Mysql.prepare

        def self.transaction(config, schema, count, &control)
          task = {
            'host' => config.param('host', :string),
            'port' => config.param('port', :integer, default: 3306),
            'username' => config.param('username', :string),
            'password' => config.param('password', :string, default: ''),
            'database' => config.param('database', :string),
            'table' => config.param('table', :string),
          }
          yield task
          {}
        end

        def self.resume(task, schema, count, &control)
          # TODO: Supporting resume
          yield(task)
          {}
        end

        def initialize(task, schema, index)
          super

          @task = task
          @schema = schema

          @mysql = self.class.connect task

          sql_schema = self.class.to_mysql_schema schema

          @mysql.query("CREATE TABLE IF NOT EXISTS #{quoted_table_name} (#{sql_schema})")
        end

        def close
          @mysql.close
        end

        def add(page)
          page.each do |record|
            # TODO: Support BULK-INSERT or LOAD INFILE
            @mysql.query("INSERT INTO #{quoted_table_name} (#{quoted_column_names}) VALUES (#{self.class.to_sql_values(record)})")
          end
        end

        private

        def quoted_table_name
          Mysql.quote @task['table']
        end

        def quoted_column_names
          @schema.names.map { |name| Mysql.quote name }.join(',')
        end

        def self.connect(task)
          Mysql.connect(task['host'], task['username'], task['password'], task['database'], task['port'])
        end

        def self.to_mysql_schema(schema)
          schema.names.zip(schema.types)
            .map { |name, type| "#{Mysql.quote(name)} #{to_sql_type(type)}" }
            .join(',')
        end

        def self.to_sql_type(type)
          case type
          when :boolean then 'BOOLEAN'
          when :long then 'BIGINT'
          when :double then 'DOUBLE'
          when :string then 'TEXT'
          when :timestamp then 'TIMESTAMP'
          else fail NotSupportedSchema, "embulk-plugin-output-mysql cannot take column type #{type}"
          end
        end

        def self.to_sql_values(values)
          values.map {|v| "'#{Mysql.quote(v.to_s)}'"}.join(",")
        end
      end

    end
  end
end
